#
# spec file for package nvidia-open-driver-G06-signed
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define hardcode_pci_list 0

%if %{undefined kernel_module_directory}
%if 0%{?usrmerged}
%define kernel_module_directory /usr/lib/modules
%else
%define kernel_module_directory /lib/modules
%endif
%endif

%if 0%{?suse_version} >= 1550 || 0%{?sle_version} >= 150400
%define compress_modules zstd
%else
%define compress_modules xz
%endif
Name:           nvidia-open-driver-G06-signed
Version:        525.116.04
Release:        150500.1.3
Summary:        NVIDIA open kernel module driver for GeForce RTX 2000 series and newer
License:        GPL-2.0-only AND MIT
Group:          System/Kernel
URL:            https://github.com/NVIDIA/open-gpu-kernel-modules/
Source0:        open-gpu-kernel-modules-%{version}.tar.gz
Source1:        my-find-supplements
Source2:        pci_ids-%{version}
Source3:        kmp-filelist
Source4:        kmp-post.sh
Source5:        kmp-postun.sh
Source6:        modprobe.nvidia.install
Source7:        preamble
Source8:        json-to-pci-id-list.py
Source9:        pci_ids-unsupported-%{version}
Source10:       pci_ids-unsupported
Source11:       pesign-copy-sources
Source12:       pesign-spec-macros
Source13:       generati-pci-table.sh
Patch0:         0001-Don-t-override-INSTALL_MOD_DIR.patch
Patch2:         persistent-nvidia-id-string.patch
Patch3:         pci-table.patch
BuildRequires:  %{kernel_module_package_buildreqs}
BuildRequires:  gcc-c++
BuildRequires:  kernel-source
BuildRequires:  kernel-syms
BuildRequires:  perl-Bootloader
BuildRequires:  pesign-obs-integration
BuildRequires:  zstd
%ifnarch aarch64
%if 0%{?sle_version} >= 120400 && !0%{?is_opensuse}
BuildRequires:  kernel-syms-azure
%endif
%endif
ExclusiveArch:  x86_64 aarch64

%if 0%{!?kmp_template_name:1}
%define kmp_template_name /usr/lib/rpm/kernel-module-subpackage
%endif
%(sed -e '/^%%post\>/ r %_sourcedir/kmp-post.sh' -e '/^%%postun\>/ r %_sourcedir/kmp-postun.sh' %kmp_template_name >%_builddir/nvidia-kmp-template)
%kernel_module_package -n %{name} -t %_builddir/nvidia-kmp-template -f %_sourcedir/kmp-filelist -p %_sourcedir/preamble

%if ! 0%{hardcode_pci_list}
## create hardware supplements for manual builds
%{load:%{SOURCE12}}
%endif

# newer rpmbuilds attach the kernel version and the major part of release to %%pci_id_file of the __kmp_supplements script
# boo#1190210
%define kbuildver %(rpm -q --queryformat '%%{VERSION}_%%{RELEASE}' kernel-syms | sed -n 's/\\(.*\\)\\.[0-9]\\{1,\\}/\\1/p')

%description
This package provides the open-source NVIDIA kernel module driver
for GeForce RTX 2000 series and newer GPUs.

%prep
%setup -q -n open-gpu-kernel-modules-%{version}
%patch0 -p1
%patch2 -p1
%if 0%{hardcode_pci_list}
%patch3 -p0
pushd kernel-open
%if 0%{?suse_version} >= 1550
sh %{SOURCE13} %{SOURCE2}
%else
sh %{SOURCE13} %{SOURCE9}
%endif
popd
%endif
set -- *
mkdir source
mv "$@" source/
mkdir obj

pushd %_sourcedir
chmod 755 my-find-supplements*
# symlink the %pci_id_file to the one, that rpmbuild generates, to enable my-find-supplement to succeed properly
# boo#1190210
ln -sv pci_ids-%{version} pci_ids-%{version}_k%{kbuildver}
popd

%build
%ifarch aarch64
# -Wall is upstream default
export CFLAGS="-Wall -mno-outline-atomics"
%endif
# kernel was compiled using a different compiler
export CC=gcc
for flavor in %{flavors_to_build}; do
        rm -rf obj/$flavor
        cp -r source obj/$flavor
	pushd obj/$flavor
	if [ -d /usr/src/linux-$flavor ]; then
	  export SYSSRC=/usr/src/linux-$flavor
	else
	  export SYSSRC=/usr/src/linux
	fi
	export SYSOUT=/usr/src/linux-obj/%_target_cpu/$flavor
        make %{?_smp_mflags} %{?linux_make_arch} modules
        popd
done

%install
export BRP_PESIGN_FILES="*.ko"
export BRP_PESIGN_COMPRESS_MODULE=%{compress_modules}
export INSTALL_MOD_PATH=%{buildroot}
export INSTALL_MOD_DIR=updates
for flavor in %{flavors_to_build}; do
	pushd obj/$flavor
	if [ -d /usr/src/linux-$flavor ]; then
	  export SYSSRC=/usr/src/linux-$flavor
	else
	  export SYSSRC=/usr/src/linux
	fi
	export SYSOUT=/usr/src/linux-obj/%_target_cpu/$flavor
        make %{?linux_make_arch} modules_install
	popd
done

%if 0%{?suse_version} >= 1550
MODPROBE_DIR=%{buildroot}/usr/lib/modprobe.d
%else
MODPROBE_DIR=%{buildroot}%{_sysconfdir}/modprobe.d
%endif

mkdir -p $MODPROBE_DIR
for flavor in %flavors_to_build; do
    cat > $MODPROBE_DIR/50-nvidia-$flavor.conf << EOF
blacklist nouveau
options nvidia-drm modeset=1
### Enable support on *all* Turing/Ampere GPUs: Alpha Quality!
%if 0%{hardcode_pci_list}
%if 0%{?suse_version} >= 1550
#options nvidia NVreg_OpenRmEnableUnsupportedGpus=1
%else
options nvidia NVreg_OpenRmEnableUnsupportedGpus=1
%endif
%else
#options nvidia NVreg_OpenRmEnableUnsupportedGpus=1
%endif
EOF
    echo -n "install nvidia " >> $MODPROBE_DIR/50-nvidia-$flavor.conf
    tail -n +3 %_sourcedir/modprobe.nvidia.install | awk '{ printf "%s ", $0 }' >> $MODPROBE_DIR/50-nvidia-$flavor.conf
# otherwise nvidia-uvm is missing in initrd and won't get loaded when nvidia
# module is loaded in initrd; so better let's load all the nvidia modules
# later ...
%if 0%{?suse_version} >= 1550
  mkdir -p %{buildroot}/usr/lib/dracut/dracut.conf.d
  cat  >   %{buildroot}/usr/lib/dracut/dracut.conf.d/60-nvidia-$flavor.conf << EOF
%else
  mkdir -p %{buildroot}/etc/dracut.conf.d
  cat  > %{buildroot}/etc/dracut.conf.d/60-nvidia-$flavor.conf << EOF
%endif
omit_drivers+=" nvidia nvidia-drm nvidia-modeset nvidia-uvm "
EOF
done

%changelog
* Tue May  9 2023 sndirsch@suse.com
- Update to version 525.116.04
* Tue Apr 25 2023 sndirsch@suse.com
-  Update to version 525.116.03
* Thu Mar 30 2023 sndirsch@suse.com
- Update to version 525.105.17
* Fri Feb 10 2023 sndirsch@suse.com
- no longer use %%hardcode_pci_list macro on sle15/Leap15, since the
  issue with generating wrong PCI ID modaliases for Supplements
  has been fixed meanwhile also in pesign-obs-integration package
  for sle15/Leap15 (boo#1207520)
* Wed Feb  8 2023 sndirsch@suse.com
- Update to version 525.89.02
* Mon Jan 30 2023 sndirsch@suse.com
- no longer use %%hardcode_pci_list macro on TW, since the issue
  with generating wrong PCI ID modaliases for Supplements
  has been fixed now in pesign-obs-integration package for TW
  (boo#1207520)
* Sun Jan 29 2023 sndirsch@suse.com
- introduce %%hardcode_pci_list macro for hardcoding PCI ID
  modaliases for Supplements; make it default to *temporarily*
  workaround boo#1207520
  * TW: enable only officially supported headless GPUs!
  * Leap/sle: enable all Turing and Ampere GPUs and set the
    appropriate modprobe.d option, so we can at least begin
    testing on sle15-sp5 ...
- adds pci-table.patch, generati-pci-table.sh
* Wed Jan 25 2023 sndirsch@suse.com
- added pesign-copy-sources, pesign-spec-macros, which are needed
  to fix repackaging step required for securebooot signing kernel
  modules (boo#1207520)
- pesign-spec-macros: used for defining RPM macros; here:
  %%define __kmp_supplements %%_sourcedir/my-find-supplements \
    %%_sourcedir/pci_ids-%%{version}
- pesign-copy-sources: sources needed by the RPM macros above;
  here: pci_ids-%%{version}, my-find-supplements
- all above needs changes for pesign-obs-integration first
  (SR#1060443)
* Tue Jan 24 2023 sndirsch@suse.com
- added conflicts to nvidia-driver-G06-kmp package (boo#1207495)
* Thu Jan 19 2023 sndirsch@suse.com
- Update to version 525.85.05
  * Fix build problems with Clang 15.0, #377 by @ptr1337
* Thu Jan 19 2023 sndirsch@suse.com
- require "group(video)"
* Thu Jan 19 2023 sndirsch@suse.com
- set options for nvidia kernel module via modprobe install script;
  no longer adjusting of nvidia's NVreg_DeviceFileGID option in
  modprobe.d file needed in %%post
* Thu Jan 19 2023 sndirsch@suse.com
- package the modprobe.d file as a sample in
  /usr/lib/nvidia-open-driver-G06-signed; then adjust GUID of
  video group for nvidia's NVreg_DeviceFileGID option and write the
  result to /etc/modprobe.d, i.e. no longer touch a packaged
  modprobe.d file below /usr
* Thu Jan 19 2023 sndirsch@suse.com
- set group id for nvidia's NVreg_DeviceFileGID option already
  during build, since it's not allowed to edit modprobe.d files
  during %%post
* Wed Jan 18 2023 sndirsch@suse.com
- added pci_ids-unsupported-525.78.01/pci_ids-unsupported as sources
  to specfile
* Wed Jan 18 2023 sndirsch@suse.com
- added empty %%changelog section to specfile
* Wed Jan 18 2023 sndirsch@suse.com
- implements jsc#PED-2658
* Mon Jan  9 2023 sndirsch@suse.com
- require kernel-firmware-nvidia-gsp-G06 (with suffix -G06!)
* Mon Jan  9 2023 sndirsch@suse.com
- Update to version 525.78.01
  * adds alpha support for
    + NVIDIA GeForce RTX 3050
    + NVIDIA GeForce RTX 4090
    + NVIDIA RTX 6000 Ada Generation
    + NVIDIA L40
    + NVIDIA GeForce RTX 4080
    + NVIDIA GeForce RTX 4070 Ti
* Wed Jan  4 2023 sndirsch@suse.com
- renamed package to nvidia-open-driver-G06-signed due to package
  redesign
* Mon Nov 28 2022 sndirsch@suse.com
- Update to version 525.60.11
* Wed Nov 23 2022 sndirsch@suse.com
- set NVreg_PreserveVideoMemoryAllocations kernel option now
  needed for GNOME Wayland (gdm) since
    commit 51181871e9db716546e9593216220389de0d8b03
    Author: Ray Strode <rstrode@redhat.com>
    Date:   Fri Mar 4 14:11:03 2022 -0500
    data: Disable wayland on nvidia if suspend is broken
* Mon Nov 14 2022 sndirsch@suse.com
- added /etc/dracut.conf.d/60-nvidia-$flavor.conf in order to
  omit nvidia modules in initrd, which apparently were added
  automatically - apart from nvidia-uvm module; related to
  boo#1173733
* Fri Nov 11 2022 sndirsch@suse.com
- Update to 525.53
  * GSP firmware is now distributed as multiple firmware files:
    this release has gsp_tu10x.bin and gsp_ad10x.bin replacing
    gsp.bin from previous releases.
    + Each file is named after a GPU architecture and supports
    GPUs from one or more architectures. This allows GSP
    firmware to better leverage each architecture's
    capabilities.
    + The .run installer will continue to install firmware to
    /lib/firmware/nvidia/<version> and the nvidia.ko kernel
    module will load the appropriate firmware for each GPU at
    runtime.
  * Add support for IBT (indirect branch tracking) on supported
    platforms, #256 by @rnd-ash
  * Return EINVAL when [failing to] allocating memory,
    [#280] by @YusufKhan-gamedev
  * Fix various typos in nvidia/src/kernel, #16 by
    @alexisgeoffrey
  * Added support for rotation in X11, Quadro Sync, Stereo,
    and YUV 4:2:0 on Turing.
* Tue Oct 18 2022 sndirsch@suse.com
- (build-)require perl-Bootloader due to pbl usage in %%post(-un)
* Tue Oct 18 2022 sndirsch@suse.com
- kmp-post.sh/kmp-postun.sh:
  * add/remove nosimplefb=1 kernel option in order to fix Linux
    console on TW kernel, which comes with simpledrm support
* Sat Oct  8 2022 sndirsch@suse.com
- Update to 515.76
  * Improved compatibility with new Linux kernel releases
  * Fixed possible excessive GPU power draw on an idle X11 or
    Wayland desktop when driving high resolutions or refresh rates
* Wed Aug  3 2022 sndirsch@suse.com
- added pci_ids-unsupported -> pci_ids-unsupported-515.65.01
  symlink to which I can refer from blog
* Tue Aug  2 2022 sndirsch@suse.com
- Update to 515.65.01
  * Collection of minor fixes to issues, #6 by @Joshua-Ashton
  * Remove unnecessary use of acpi_bus_get_device().
* Tue Jul 19 2022 gmbr3@opensuse.org
- Force CC to gcc to avoid compiler differs warnings
* Tue Jul 19 2022 sndirsch@suse.com
- json-to-pci-id-list.py
  * added new Python script by Tamara Schmitz which is being used
  now to parse supported-gpus.json in order to create the PCI ID
  list 'pci_ids-unsupported-<version>'
* Mon Jul 18 2022 sndirsch@suse.com
- pci_ids-unsupported-515.57
  * currently unsupported devices (Turing and later, which needs
    NVreg_OpenRmEnableUnsupportedGpus=1 kernel option; Alpha Quality!)
* Thu Jul 14 2022 sndirsch@suse.com
- define %%kernel_module_directory macro on older distros in order
  to fix build on Leap 15.3(GA)/sle 15 SP3(GA)
* Thu Jun 30 2022 sndirsch@suse.com
- update to 515.57
  * Backtick is deprecated, #273 by @arch-user-france1
* Fri Jun 17 2022 gmbr3@opensuse.org
- Revert last change and use default CA [pesign-obs-integration
  and kernel-source docs are incorrect]
* Thu Jun 16 2022 gmbr3@opensuse.org
- Create uefi cert package
* Wed Jun 15 2022 gmbr3@opensuse.org
- Sign the kernel modules and compress them
* Wed Jun 15 2022 sndirsch@suse.com
- don't buildrequire azure on aarch64; no such kernel flavor
  available on aarch64
* Thu Jun  9 2022 sndirsch@suse.com
- move cuda-drivers = %%version provide to nvidia-computeG0X, which
  then again requires nvidia-gfxG0X-kmp or nvidia-open-gfxG0X-kmp
  respectively
* Wed Jun  8 2022 sndirsch@suse.com
- preamble: provide cuda-drivers = %%version, so you can just
  install 'cuda' meta package to install CUDA stack
  (cuda --> cuda-11-7 --> cuda-runtime-11-7 --> cuda-drivers >= 515.43.04)
  from https://developer.download.nvidia.com/compute/cuda/repos/opensuse15/x86_64/
* Tue Jun  7 2022 sndirsch@suse.com
- preamble: added Conflicts to nvidia-gfxG06-kmp
* Tue May 31 2022 sndirsch@suse.com
- update to 515.48.07
  * List of compatible GPUs in README.md.
  * Fix various README capitalizations, #8 by @lx-is
  * Automatically tag bug report issues, #15 by @thebeanogamer
  * Improve conftest.sh Script, #37 by @Nitepone
  * Update HTTP link to HTTPS, #101 by @alcaparra
  * moved array sanity check to before the array access, #117 by @RealAstolfo
  * Fixed some typos, #122 by @FEDOyt
  * Fixed capitalization, #123 by @keroeslux
  * Fix typos in NVDEC Engine Descriptor, #126 from @TrickyDmitriy
  * Extranous apostrohpes in a makefile script [sic], #14 by @kiroma
  * HDMI no audio @ 4K above 60Hz, #75 by @adolfotregosa
  * dp_configcaps.cpp:405: array index sanity check in wrong place?, #110 by @dcb314
  * NVRM kgspInitRm_IMPL: missing NVDEC0 engine, cannot initialize GSP-RM, #116 by @kfazz
  * ERROR: modpost: "backlight_device_register" [...nvidia-modeset.ko] undefined, #135 by @sndirsch
  * aarch64 build fails, #151 by @frezbo
- supersedes backlight-workaround.patch
* Thu May 19 2022 gmbr3@opensuse.org
- Set mno-outline-atomics to fix libgcc issues on aarch64
* Thu May 19 2022 sndirsch@suse.com
- preamble: require kernel-firmware-nvidia-gsp package for nvidia's
  versioned "gsp.bin" fimrware
* Thu May 19 2022 sndirsch@suse.com
- enable build also for aarch64
* Thu May 19 2022 sndirsch@suse.com
- improved summary and description
* Thu May 19 2022 sndirsch@suse.com
- modprobe.nvidia.install
  * install script when the "nvidia" module is being loaded; needed
    for creating devices
- specfile
  * filled modprobe.d config file (options and install script)
* Thu May 19 2022 sndirsch@suse.com
- kmp-post.sh/kmp-postun.sh
  * additional code for %%post/%%postun scriptlets to create nvidia
    devices
- specfile: adjust %%kernel_module_package macro appropriately
* Tue May 17 2022 sndirsch@suse.com
- added kmp-filelist
- adding an empty modprobe.d config file (to be filled later)
* Tue May 17 2022 sndirsch@suse.com
- added hardware supplements
  * added my-find-supplements, pci_ids-515.43.04
  * changes in specfile to generate the hardware supplements from
    pci_ids-515.43.04 via my-find-supplements
* Tue May 17 2022 tiwai@suse.com
- Set SYSSRC and SYSOUT for building with proper kernel source tree
  for each flavor
- Exclusive build for x86_64
- backlight-workaround.patch:
  workaround for a build failure with azure flavor
- persistent-nvidia-id-string.patch:
  make persistent nvidia id string for reproducible builds
* Fri May 13 2022 sndirsch@suse.com
- switched to a known license to fix build:  MIT/GPLv2 --> GPL-2.0 and MIT
* Fri May 13 2022 patrik.jakobsson@suse.com
- 0001-Don-t-override-INSTALL_MOD_DIR.patch
  * Make sure INSTALL_MOD_DIR from our spec-file is honoured
- Make the default flavor build. Not sure how to get other flavors to build.
* Thu May 12 2022 sndirsch@suse.com
- created package
  * build still fails completely
  * more things are missing
